@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # <xbar.title>Meta Package Manager</xbar.title>
-# <xbar.version>v5.7.0</xbar.version>
+# <xbar.version>v5.8.0</xbar.version>
 # <xbar.author>Kevin Deldycke</xbar.author>
 # <xbar.author.github>kdeldycke</xbar.author.github>
 # <xbar.desc>List outdated packages and manage upgrades.</xbar.desc>
@@ -27,6 +27,8 @@ quite resource intensive, and Homebrew might hit GitHub's API calls quota.
 - This is in progress for SwiftBar at:
   https://github.com/swiftbar/SwiftBar/issues/160
 """
+from __future__ import annotations
+
 import sys
 
 python_min_version = (3, 7, 3)
@@ -37,8 +39,10 @@ required to work with Python 3.7.3 or newer.
 """
 
 
-def v_to_str(version_tuple):
+def v_to_str(version_tuple: tuple[int, ...] | None) -> str:
     """Transforms into a string a tuple of integers representing a version."""
+    if not version_tuple:
+        return "None"
     return ".".join(map(str, version_tuple))
 
 
@@ -77,7 +81,7 @@ class MPMPlugin:
     """Mpm v5.0.0 was the first version taking care of the complete layout rendering."""
 
     @staticmethod
-    def extended_environment():
+    def extended_environment() -> dict[str, str]:
         """Returns a tweaked environment extending global path to find non-default
         system-wide binaries.
 
@@ -108,7 +112,7 @@ class MPMPlugin:
         return env_copy
 
     @staticmethod
-    def getenv_str(var, default=None):
+    def getenv_str(var, default: str | None = None) -> str | None:
         """Utility to get environment variables.
 
         Note that all environment variables are strings. Always returns a lowered-case
@@ -120,7 +124,7 @@ class MPMPlugin:
         return str(value).lower()
 
     @staticmethod
-    def getenv_bool(var, default=False):
+    def getenv_bool(var, default: bool = False) -> bool:
         """Utility to normalize boolean environment variables.
 
         Relies on ``configparser.RawConfigParser.BOOLEAN_STATES`` to translate strings into boolean. See:
@@ -132,7 +136,9 @@ class MPMPlugin:
         return RawConfigParser.BOOLEAN_STATES[value]
 
     @staticmethod
-    def normalize_params(font_string, valid_ids={"color", "font", "size"}):
+    def normalize_params(
+        font_string: str, valid_ids: set[str] = {"color", "font", "size"}
+    ) -> str:
         valid_params = {}
         for param in font_string.split():
             param_id, param_value = param.split("=", 1)
@@ -141,7 +147,7 @@ class MPMPlugin:
         return " ".join(f"{k}={v}" for k, v in valid_params.items())
 
     @cached_property
-    def table_rendering(self):
+    def table_rendering(self) -> bool:
         """Aligns package names and versions, like a table, for easier visual parsing.
 
         If ``True``, will aligns all items using a fixed-width font.
@@ -149,29 +155,31 @@ class MPMPlugin:
         return self.getenv_bool("VAR_TABLE_RENDERING", True)
 
     @cached_property
-    def default_font(self):
-        """Make it easier to change font, sizes and colors of the output."""
-        return self.normalize_params(self.getenv_str("VAR_DEFAULT_FONT", ""))
-
-    @cached_property
-    def monospace_font(self):
+    def default_font(self) -> str:
         """Make it easier to change font, sizes and colors of the output."""
         return self.normalize_params(
-            self.getenv_str("VAR_MONOSPACE_FONT", "font=Menlo size=12")
+            self.getenv_str("VAR_DEFAULT_FONT", "")  # type: ignore
         )
 
     @cached_property
-    def error_font(self):
+    def monospace_font(self) -> str:
+        """Make it easier to change font, sizes and colors of the output."""
+        return self.normalize_params(
+            self.getenv_str("VAR_MONOSPACE_FONT", "font=Menlo size=12")  # type: ignore
+        )
+
+    @cached_property
+    def error_font(self) -> str:
         """Error font is based on monospace font."""
         return self.normalize_params(f"{self.monospace_font} color=red")
 
     @cached_property
-    def is_swiftbar(self):
+    def is_swiftbar(self) -> bool:
         """SwiftBar is kind enough to tell us about its presence."""
         return self.getenv_bool("SWIFTBAR")
 
     @staticmethod
-    def locate_bin(*bin_names):
+    def locate_bin(*bin_names: str) -> str | None:
         """Find the location of an executable binary on the system.
 
         Provides as many binary names as you need, the first one found will be returned.
@@ -181,19 +189,20 @@ class MPMPlugin:
             path = which(name)
             if path:
                 return path
+        return None
 
     @cached_property
-    def python_path(self):
+    def python_path(self) -> str:
         """Returns the system's Python binary path.
 
         This plugin being run from Python, we have the one called by Xbar/SwiftBar to
         fallback to (i.e. ``sys.executable``). But before that, we attempt to locate it
         by respecting the environment variables.
         """
-        return self.locate_bin("python", "python3", sys.executable)
+        return self.locate_bin("python", "python3", sys.executable)  # type: ignore
 
     @cached_property
-    def mpm_exec(self):
+    def mpm_exec(self) -> tuple[str, ...]:
         """Search for mpm execution alternatives, either direct ``mpm`` call or as an
         executable Python module."""
         # XXX Local debugging and development.
@@ -203,9 +212,11 @@ class MPMPlugin:
             return (mpm_exec,)
         return self.python_path, "-m", "meta_package_manager"
 
-    def check_mpm(self):
+    def check_mpm(
+        self,
+    ) -> tuple[bool, tuple[int, ...] | None, bool, str | Exception | None]:
         """Test-run mpm execution and extract its version."""
-        error = None
+        error: str | Exception | None = None
         try:
             process = run(
                 (*self.mpm_exec, "--version"), capture_output=True, encoding="utf-8"
@@ -221,25 +232,25 @@ class MPMPlugin:
         if not process.returncode and not error:
             installed = True
             # Is mpm too old?
-            version_string = (
-                re.compile(r".*\s+(?P<version>[0-9\.]+)$", re.MULTILINE)
-                .search(process.stdout)
-                .groupdict()["version"]
+            match = re.compile(r".*\s+(?P<version>[0-9\.]+)$", re.MULTILINE).search(
+                process.stdout
             )
-            mpm_version = tuple(map(int, version_string.split(".")))
-            if mpm_version >= self.mpm_min_version:
-                up_to_date = True
+            if match:
+                version_string = match.groupdict()["version"]
+                mpm_version = tuple(map(int, version_string.split(".")))
+                if mpm_version >= self.mpm_min_version:
+                    up_to_date = True
 
         return installed, mpm_version, up_to_date, error
 
     @staticmethod
-    def pp(*args):
+    def pp(*args: str) -> None:
         """Print one menu-line with the Xbar/SwiftBar dialect.
 
         First argument is the menu-line label, separated by a pipe to all other non-
         empty parameters, themselves separated by a space.
         """
-        line = []
+        line: list[str] = []
         for param in args:
             if param:
                 if len(line) == 1:
@@ -248,12 +259,12 @@ class MPMPlugin:
         print(*line, sep=" ")
 
     @staticmethod
-    def print_error_header():
+    def print_error_header() -> None:
         """Generic header for blocking error."""
         MPMPlugin.pp("❗️", "dropdown=false")
         print("---")
 
-    def print_error(self, message, submenu=""):
+    def print_error(self, message: str | Exception, submenu: str = "") -> None:
         """Print a formatted error line by line.
 
         A red, fixed-width font is used to preserve traceback and exception layout.
@@ -269,13 +280,14 @@ class MPMPlugin:
                 "symbolize=false" if self.is_swiftbar else "",
             )
 
-    def print_menu(self):
+    def print_menu(self) -> None:
         """Print the main menu."""
         mpm_installed, _, mpm_up_to_date, error = self.check_mpm()
         if not mpm_installed or not mpm_up_to_date:
             self.print_error_header()
-            self.print_error(error)
-            print("---")
+            if error:
+                self.print_error(error)
+                print("---")
             action_msg = "Install" if not mpm_installed else "Upgrade"
             min_version_str = v_to_str(self.mpm_min_version)
             self.pp(
