@@ -21,12 +21,12 @@ import os
 import re
 import sys
 from contextlib import nullcontext
-from dataclasses import InitVar, asdict, dataclass
+from dataclasses import asdict, dataclass
 from enum import Enum
 from pathlib import Path
 from shutil import which
 from textwrap import dedent, indent, shorten
-from typing import ContextManager, Iterator, Optional, Tuple
+from typing import ContextManager, Iterator
 from unittest.mock import patch
 
 if sys.version_info >= (3, 8):
@@ -65,7 +65,7 @@ Operations = Enum(
         "cleanup",
     ),
 )
-"""Recognize operation IDs that are implemented by package manager with their specific CLI invocation.
+"""Recognized operation IDs that are implemented by package manager with their specific CLI invocation.
 
 Each operation has its own CLI subcommand.
 """
@@ -114,9 +114,6 @@ class Package:
 
     name: str | None = None
 
-    label: str | None = None
-    """Label is used in *Bar plugin to display user-friendly entries."""
-
     description: str | None = None
 
     installed_version: TokenizedString | str | None = None
@@ -126,23 +123,11 @@ class Package:
     normalized into ``TokenizedString``.
     """
 
-    upgrade_cli: tuple[str, ...] | None = None
-
-    manager: InitVar[PackageManager] = None
-    """Init-only variable user to populate package metadata requiring a lookup from the manager."""
-
-    def __post_init__(self, manager) -> None:
-        self.label = self.name if self.name else self.id
-
+    def __post_init__(self) -> None:
         # Make sure version strings are parsed into proper objects.
         self.installed_version = parse_version(self.installed_version)
         self.latest_version = parse_version(self.latest_version)
 
-        try:
-            self.upgrade_cli = manager.upgrade_one_cli(self.id)
-        except NotImplementedError:
-            pass
-
 
 def packages_asdict(packages: Iterator[Package], keep_fields: tuple[str, ...]):
     """Returns a list of packages casted to a ``dict`` with only a subset of its
@@ -281,7 +266,7 @@ class PackageManager(metaclass=MetaPackageManager):
     cli_errors: list[CLIError]
     """Accumulate all CLI errors encountered by the package manager."""
 
-    package_class: type[Package] = Package
+    package: type[Package] = Package
     """The dataclass to use to produce Package objects from the manager."""
 
     def __init__(self) -> None:
@@ -678,11 +663,6 @@ class PackageManager(metaclass=MetaPackageManager):
 
         return output
 
-    def package(self, *args, **kwargs) -> Package:
-        """Produce a ``Package`` object with provided attributes."""
-        kwargs["manager"] = self
-        return self.package_class(*args, **kwargs)
-
     @property
     def installed(self) -> Iterator[Package]:
         """List packages currently installed on the system.
@@ -769,10 +749,10 @@ class PackageManager(metaclass=MetaPackageManager):
             # Report the package as matching.
             yield match
 
-    def install(self, package_id: str) -> str:
+    def install(self, package_id: str, version: str | None = None) -> str:
         """Install one package and one only.
 
-        Optional. Will be simply skipped by :program:`mpm` if not implemented.
+        Allows a specific ``version`` to be provided.
         """
         raise NotImplementedError
 
@@ -780,11 +760,16 @@ class PackageManager(metaclass=MetaPackageManager):
         """Returns the complete CLI to upgrade all outdated packages on the system."""
         raise NotImplementedError
 
-    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
-        """Returns the complete CLI to upgrade one package and one only."""
+    def upgrade_one_cli(
+        self, package_id: str, version: str | None = None
+    ) -> tuple[str, ...]:
+        """Returns the complete CLI to upgrade one package and one only.
+
+        Allows a specific ``version`` to be provided.
+        """
         raise NotImplementedError
 
-    def upgrade(self, package_id: str | None = None) -> str:
+    def upgrade(self, package_id: str | None = None, version: str | None = None) -> str:
         """Perform an upgrade of either all or one package.
 
         Executes the CLI provided by either
@@ -800,7 +785,7 @@ class PackageManager(metaclass=MetaPackageManager):
         See for example the case of :py:meth:`meta_package_manager.managers.pip.Pip.upgrade_one_cli`.
         """
         if package_id:
-            cli = self.upgrade_one_cli(package_id)
+            cli = self.upgrade_one_cli(package_id, version=version)
 
         else:
             try:
