@@ -58,6 +58,7 @@ from .output import (
     print_table,
 )
 from .pool import pool
+from .specifier import Solver, Specifier
 
 # Subcommand sections.
 EXPLORE = Section("Explore subcommands")
@@ -139,8 +140,8 @@ def bar_plugin_path(ctx, param, value):
         type=Choice(pool.all_manager_ids, case_sensitive=False),
         multiple=True,
         help="Restrict subcommand to a subset of managers. Repeat to "
-        "select multiple managers. The order in which options are provided defines the "
-        "order in which subcommands will process them.",
+        "select multiple managers. The order is preserved for priority-sensitive "
+        "subcommands.",
     ),
     option(
         "-e",
@@ -169,7 +170,7 @@ def bar_plugin_path(ctx, param, value):
     *single_manager_selectors(),
 )
 @option_group(
-    "Manager's options",
+    "Manager options",
     option(
         "--ignore-auto-updates/--include-auto-updates",
         default=True,
@@ -242,7 +243,7 @@ def mpm(
     sort_by,
     stats,
 ):
-    """Common CLI options and behavior for managers."""
+    """Common CLI options for subcommands."""
 
     # Update the list of selected managers with single selectors.
     if ctx.obj:
@@ -490,25 +491,15 @@ def outdated(ctx, plugin_output):
     """List available package upgrades and their versions for each manager."""
     # Build-up a global list of outdated packages per manager.
     outdated_data = {}
-    fields = [
+    fields = (
         "id",
         "name",
         "installed_version",
         "latest_version",
-    ]
-    if plugin_output:
-        fields.extend(["label", "upgrade_cli"])
+    )
 
     for manager in ctx.obj.selected_managers(implements_operation=Operations.outdated):
-
-        packages = tuple(packages_asdict(manager.outdated, tuple(fields)))
-
-        # Re-render upgrade CLI for plugin consumption.
-        if plugin_output:
-            for info in packages:
-                info["upgrade_cli"] = BarPluginRenderer.render_cli(
-                    info["upgrade_cli"], plugin_format=plugin_output
-                )
+        packages = tuple(packages_asdict(manager.outdated, fields))
 
         outdated_data[manager.id] = {
             "id": manager.id,
@@ -516,20 +507,6 @@ def outdated(ctx, plugin_output):
             "packages": packages,
         }
 
-        # Only include full-upgrade CLI for plugin.
-        if plugin_output and packages:
-            try:
-                upgrade_all_cli = manager.upgrade_all_cli()
-            except NotImplementedError:
-                # Fallback on mpm itself which is capable of simulating a full upgrade.
-                logger.warning(f"{manager.id} does not implement upgrade_all_cli.")
-                mpm_exec = bar_plugin.MPMPlugin().mpm_exec
-                upgrade_all_cli = (*mpm_exec, f"--{manager.id}", "upgrade", "--all")
-                logger.debug(f"Fallback to direct mpm call: {upgrade_all_cli}")
-            outdated_data[manager.id]["upgrade_all_cli"] = BarPluginRenderer.render_cli(
-                upgrade_all_cli, plugin_format=plugin_output
-            )
-
         # Serialize errors at the last minute to gather all we encountered.
         outdated_data[manager.id]["errors"] = list(
             {expt.error for expt in manager.cli_errors}
@@ -695,57 +672,110 @@ def search(ctx, extended, exact, refilter, query):
 
 
 @mpm.command(short_help="Install a package.", section=MAINTENANCE)
-@argument("package_id", type=STRING, required=True)
+@argument(
+    "packages_specs",
+    type=STRING,
+    nargs=-1,
+    required=True,
+    help="A mix of plain <package_id>, simple <package_id@version> specifiers or full <pkg:npm/left-pad> purls.",
+)
 # TODO: add a --force/--reinstall flag
 @pass_context
-def install(ctx, package_id):
-    """Install the provided package ID using one of the selected manager."""
+def install(ctx, packages_specs):
+    """Install one or more packages.
+
+    Installation will proceed first with packages unambiguously tied to a manager. You can have an
+    influence on that with more precise package specifiers (like purl) and/or tighter selection of managers.
+
+    For other untied packages, mpm will try to find the best manager to install it with. Their installation
+    will be attempted with each manager, in the order they were selected. If we have the certainty, by the way
+    of a search operation, that this package is not available from this manager, we'll skip the installation
+    and try the next available manager.
+    """
     # Cast generator to tuple because of reuse.
     selected_managers = tuple(
         ctx.obj.selected_managers(implements_operation=Operations.install)
     )
+    manager_ids = tuple(manager.id for manager in selected_managers)
+    if len(manager_ids) > 1:
+        logger.info(
+            f"Installation priority: {' > '.join(map(theme.invoked_command, manager_ids))}"
+        )
 
-    logger.info(
-        f"Package manager order: {', '.join([m.id for m in selected_managers])}"
-    )
+    solver = Solver(packages_specs, manager_priority=manager_ids)
+    packages_per_managers = solver.resolve_specs_group_by_managers()
 
-    for manager in selected_managers:
-        logger.debug(f"Try to install {package_id} with {manager.id}.")
-
-        # Is the package available on this manager?
-        matches = None
-        try:
-            matches = tuple(
-                manager.refiltered_search(extended=False, exact=True, query=package_id)
-            )
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement search operation.")
-            logger.info(
-                f"{package_id} existence unconfirmed, try to directly install it..."
-            )
-        else:
-            if not matches:
-                logger.warning(f"No {package_id} package found on {manager.id}.")
-                continue
-            # Prevents any incomplete or bad implementation of exact search.
-            if len(matches) != 1:
-                raise ValueError("Exact search returned multiple packages.")
-
-        # Allow install subcommand to fail to have the opportunity to catch the CLIError exception and print
-        # a comprehensive message.
-        with patch.object(manager, "stop_on_error", True):
+    # Install all packages deterministiccaly tied to a specific manager.
+    for manager_id, package_specs in packages_per_managers.items():
+        if not manager_id:
+            continue
+        for spec in package_specs:
             try:
-                logger.info(f"Install {package_id} package from {manager.id}...")
-                output = manager.install(package_id)
+                logger.info(
+                    f"Install {spec} package with {theme.invoked_command(manager_id)}..."
+                )
+                manager = pool.get(manager_id)
+                output = manager.install(spec.package_id, version=spec.version)
             except NotImplementedError:
-                logger.warning(f"{manager.id} does not implement install operation.")
-                continue
-            except CLIError:
-                logger.warning(f"Could not install {package_id} with {manager.id}.")
+                logger.warning(
+                    f"{theme.invoked_command(manager_id)} does not implement install operation."
+                )
                 continue
+            echo(output)
 
-        echo(output)
-        ctx.exit()
+    unmatched_packages = packages_per_managers.get(None, [])
+    for spec in unmatched_packages:
+        for manager in selected_managers:
+            logger.debug(
+                f"Try to install {spec} with {theme.invoked_command(manager.id)}."
+            )
+
+            # Is the package available on this manager?
+            matches = None
+            try:
+                matches = tuple(
+                    manager.refiltered_search(
+                        extended=False, exact=True, query=spec.package_id
+                    )
+                )
+            except NotImplementedError:
+                logger.warning(
+                    f"{theme.invoked_command(manager.id)} does not implement search operation."
+                )
+                logger.info(
+                    f"{spec.package_id} existence unconfirmed, try to directly install it..."
+                )
+            else:
+                if not matches:
+                    logger.warning(
+                        f"No {spec.package_id} package found on {theme.invoked_command(manager.id)}."
+                    )
+                    continue
+                # Prevents any incomplete or bad implementation of exact search.
+                if len(matches) != 1:
+                    raise ValueError("Exact search returned multiple packages.")
+
+            # Allow install subcommand to fail to have the opportunity to catch the CLIError exception and print
+            # a comprehensive message.
+            with patch.object(manager, "stop_on_error", True):
+                try:
+                    logger.info(
+                        f"Install {spec} package with {theme.invoked_command(manager.id)}..."
+                    )
+                    output = manager.install(spec.package_id, version=spec.version)
+                except NotImplementedError:
+                    logger.warning(
+                        f"{theme.invoked_command(manager.id)} does not implement install operation."
+                    )
+                    continue
+                except CLIError:
+                    logger.warning(
+                        f"Could not install {spec} with {theme.invoked_command(manager.id)}."
+                    )
+                    continue
+
+            echo(output)
+            ctx.exit()
 
 
 @mpm.command(aliases=["update"], short_help="Upgrade packages.", section=MAINTENANCE)
@@ -757,132 +787,161 @@ def install(ctx, package_id):
     help="Upgrade all outdated packages. "
     "Will make the command ignore package IDs provided as parameters.",
 )
-@argument("package_ids", type=STRING, nargs=-1)
+@argument(
+    "packages_specs",
+    type=STRING,
+    nargs=-1,
+    help="A mix of plain <package_id>, simple <package_id@version> specifiers or full <pkg:npm/left-pad> purls.",
+)
 @pass_context
-def upgrade(ctx, all, package_ids):
+def upgrade(ctx, all, packages_specs):
     """Upgrade one or more outdated packages.
 
-    Defaults to upgrading all outdated package if none provided as arguments (i.e. assumes
-    -A/--all if no [PACKAGE_IDS]).
+    All outdated package will be upgraded by default if no specifiers are provided as arguments.
+    I.e. assumes -A/--all option if no [PACKAGES_SPECS]....
 
-    Upgrade will only proceed if no ambiguity is uncovered. Packages recognized by multiple
-    managers will be skipped. You can remove that ambiguity by carefully specifying the subset of
-    managers to consider for upgrade.
+    Packages recognized by multiple managers will be upgraded with each of them. You can fine-tune
+    this behavior with more precise package specifiers (like purl) and/or tighter selection of managers.
 
-    Unknown packages will be skipped.
+    Packages unrecognized by any selected manager will be skipped.
     """
-    if not all and not package_ids:
+    if not all and not packages_specs:
         logger.warning("No package provided, assume -A/--all option.")
         all = True
 
-    # Deduplicate entries.
-    package_ids = set(package_ids)
-
     # Full upgrade.
     if all:
-        if package_ids:
-            logger.debug(
-                f"Ignore provided {', '.join(sorted(package_ids))} packages and proceed to a full upgrade..."
+        if packages_specs:
+            # Deduplicate and sort specifiers for terseness.
+            logger.warning(
+                f"Ignore {', '.join(sorted(set(packages_specs)))} specifiers and proceed to a full upgrade..."
             )
         for manager in ctx.obj.selected_managers(
             implements_operation=Operations.upgrade_all
         ):
-            logger.info(f"Upgrade all outdated packages from {manager.id}...")
+            logger.info(
+                f"Upgrade all outdated packages from {theme.invoked_command(manager.id)}..."
+            )
             output = manager.upgrade()
             if output:
                 logger.info(output)
         ctx.exit()
 
-    # Specific list of package to upgrade has been requested. We need to
-    # validate them before proceeding.
+    # Cast generator to tuple because of reuse.
+    selected_managers = tuple(
+        ctx.obj.selected_managers(implements_operation=Operations.upgrade)
+    )
+    manager_ids = tuple(manager.id for manager in selected_managers)
 
-    # For each package, we list the managers they were sourced from.
-    package_sources = {}
-    for manager in ctx.obj.selected_managers(implements_operation=Operations.upgrade):
-        for package in manager.installed:
-            if package.id in package_ids:
-                package_sources.setdefault(package.id, set()).add(manager.id)
-    logger.debug(f"Managers sourcing each package: {package_sources}")
+    # Get the subset of selected managers that are implementing the installed operation, so we
+    # can query it and know if a package has been installed with it.
+    sourcing_managers = tuple(
+        ctx.obj.selected_managers(
+            keep=manager_ids, implements_operation=Operations.installed
+        )
+    )
 
-    for package_id in package_ids:
+    solver = Solver(packages_specs, manager_priority=manager_ids)
+    for package_id, spec in solver.resolve_package_specs():
 
-        # Skip unrecognized packages.
-        if package_id not in package_sources:
+        source_manager_ids = set()
+        # Use the manager from the spec.
+        if spec.manager_id:
+            source_manager_ids.add(spec.manager_id)
+        # Package is not bound to a manager by the user's specifiers.
+        else:
+            logger.debug(
+                f"{spec} not tied to a manager. Search all managers recognizing it."
+            )
+            # Find all the managers that have the package installed.
+            for manager in sourcing_managers:
+                if package_id in map(attrgetter("id"), manager.installed):
+                    logger.debug(
+                        f"{package_id} has been installed with {theme.invoked_command(manager.id)}."
+                    )
+                    source_manager_ids.add(manager.id)
+
+        if not source_manager_ids:
             logger.error(
-                f"{package_id} is not recognized by any of the selected manager."
-                f" Skip its upgrade."
+                f"{package_id} is not recognized by any of the selected manager. Skip it."
             )
             continue
 
-        # A package to upgrade that was sourced from multiple managers leads to undefined
-        # behavior. What should we do in this case? Upgrade the package with each manager? Only the first one? A random one?
-        # We choose to play it safe and simply report this ambiguous situation to
-        # the user and skip the package.
-        managers = package_sources[package_id]
-        if len(managers) > 1:
-            logger.error(
-                f"{package_id} was sourced from multiple managers: {', '.join(sorted(managers))}."
-                " Skip its upgrade."
-            )
-            continue
-
-        assert len(managers) == 1
-        manager_id = managers.pop()
-        manager = pool.get(manager_id)
-        logger.info(f"Proceed to upgrade {package_id} with {manager_id}...")
-        output = manager.upgrade(package_id)
-        if output:
-            logger.info(output)
+        logger.info(
+            f"Upgrade {package_id} with {', '.join(map(theme.invoked_command, sorted(source_manager_ids)))}"
+        )
+        for manager_id in source_manager_ids:
+            manager = pool.get(manager_id)
+            output = manager.upgrade(package_id, version=spec.version)
+            if output:
+                logger.info(output)
 
 
 @mpm.command(aliases=["uninstall"], short_help="Remove a package.", section=MAINTENANCE)
-@argument("package_id", type=STRING, required=True)
+@argument(
+    "packages_specs",
+    type=STRING,
+    nargs=-1,
+    required=True,
+    help="A mix of plain <package_id>, simple <package_id@version> specifiers or full <pkg:npm/left-pad> purls.",
+)
 @pass_context
-def remove(ctx, package_id):
-    """Remove the provided package using one of the selected manager."""
+def remove(ctx, packages_specs):
+    """Remove one or more packages.
+
+    Packages recognized by multiple managers will be remove with each of them. You can fine-tune
+    this behavior with more precise package specifiers (like purl) and/or tighter selection of managers.
+
+    Packages unrecognized by any selected manager will be skipped.
+    """
     # Cast generator to tuple because of reuse.
     selected_managers = tuple(
         ctx.obj.selected_managers(implements_operation=Operations.remove)
     )
+    manager_ids = tuple(manager.id for manager in selected_managers)
 
-    logger.info(
-        f"Package manager order: {', '.join([m.id for m in selected_managers])}"
+    # Get the subset of selected managers that are implementing the installed operation, so we
+    # can query it and know if a package has been installed with it.
+    sourcing_managers = tuple(
+        ctx.obj.selected_managers(
+            keep=manager_ids, implements_operation=Operations.installed
+        )
     )
 
-    for manager in selected_managers:
-        logger.debug(f"Try to remove {package_id} with {manager.id}.")
+    solver = Solver(packages_specs, manager_priority=manager_ids)
+    for package_id, spec in solver.resolve_package_specs():
 
-        # Is the package installed with this manager?
-        installed = set()
-        try:
-            installed = map(attrgetter("id"), manager.installed)
-        except NotImplementedError:
-            logger.warning(f"{manager.id} does not implement installed operation.")
-            logger.info(
-                f"{package_id} existence unconfirmed, try to directly remove it..."
-            )
+        source_manager_ids = set()
+        # Use the manager from the spec.
+        if spec.manager_id:
+            source_manager_ids.add(spec.manager_id)
+        # Package is not bound to a manager by the user's specifiers.
         else:
-            match = package_id in installed
-            if not match:
-                logger.warning(f"{package_id} was not installed by {manager.id}.")
-                continue
-            logger.info(f"{package_id} has been installed by {manager.id}.")
+            logger.debug(
+                f"{spec} not tied to a manager. Search all managers recognizing it."
+            )
+            # Find all the managers that have the package installed.
+            for manager in sourcing_managers:
+                if package_id in map(attrgetter("id"), manager.installed):
+                    logger.debug(
+                        f"{package_id} has been installed with {theme.invoked_command(manager.id)}."
+                    )
+                    source_manager_ids.add(manager.id)
 
-        # Allow remove subcommand to fail to have the opportunity to catch the CLIError exception and print
-        # a comprehensive message.
-        with patch.object(manager, "stop_on_error", True):
-            try:
-                logger.info(f"Remove {package_id} package with {manager.id}...")
-                output = manager.remove(package_id)
-            except NotImplementedError:
-                logger.warning(f"{manager.id} does not implement remove operation.")
-                continue
-            except CLIError:
-                logger.warning(f"Could not remove {package_id} with {manager.id}.")
-                continue
+        if not source_manager_ids:
+            logger.error(
+                f"{package_id} is not recognized by any of the selected manager. Skip it."
+            )
+            continue
 
-        echo(output)
-        ctx.exit()
+        logger.info(
+            f"Remove {package_id} with {', '.join(map(theme.invoked_command, sorted(source_manager_ids)))}"
+        )
+        for manager_id in source_manager_ids:
+            manager = pool.get(manager_id)
+            output = manager.remove(package_id)
+            if output:
+                logger.info(output)
 
 
 @mpm.command(short_help="Sync local package info.", section=MAINTENANCE)
@@ -890,7 +949,7 @@ def remove(ctx, package_id):
 def sync(ctx):
     """Sync local package metadata and info from external sources."""
     for manager in ctx.obj.selected_managers(implements_operation=Operations.sync):
-        logger.info(f"Sync {manager.id} package info...")
+        logger.info(f"Sync {theme.invoked_command(manager.id)} package info...")
         manager.sync()
 
 
@@ -899,7 +958,7 @@ def sync(ctx):
 def cleanup(ctx):
     """Cleanup local data, temporary artifacts and removes orphaned dependencies."""
     for manager in ctx.obj.selected_managers(implements_operation=Operations.cleanup):
-        logger.info(f"Cleanup {manager.id}...")
+        logger.info(f"Cleanup {theme.invoked_command(manager.id)}...")
         manager.cleanup()
 
 
@@ -1032,7 +1091,7 @@ def backup(ctx, overwrite, merge, update_version, toml_path):
         for manager in ctx.obj.selected_managers(
             implements_operation=Operations.installed
         ):
-            logger.info(f"Dumping packages from {manager.id}...")
+            logger.info(f"Dumping packages from {theme.invoked_command(manager.id)}...")
 
             packages = tuple(packages_asdict(manager.installed, fields))
 
@@ -1073,10 +1132,7 @@ def backup(ctx, overwrite, merge, update_version, toml_path):
 @argument("toml_files", type=File("r"), required=True, nargs=-1)
 @pass_context
 def restore(ctx, toml_files):
-    """Read TOML files then install or upgrade each package referenced in them.
-
-    Version specified in the TOML file is ignored in the current implementation.
-    """
+    """Read TOML files then install or upgrade each package referenced in them."""
     for toml_input in toml_files:
 
         is_stdin = isinstance(toml_input, TextIOWrapper)
@@ -1098,12 +1154,17 @@ def restore(ctx, toml_files):
             implements_operation=Operations.install
         ):
             if manager.id not in doc:
-                logger.warning(f"No [{manager.id}] section found.")
+                logger.warning(
+                    f"No [{theme.invoked_command(manager.id)}] section found."
+                )
                 continue
-            logger.info(f"Restore {manager.id} packages...")
+            logger.info(f"Restore {theme.invoked_command(manager.id)} packages...")
             for package_id, version in doc[manager.id].items():
+                spec = Specifier(package_id=package_id, version=version)
                 # Let the command fail if the manager doesn't implement the
                 # install operation.
-                logger.info(f"Install {package_id} package from {manager.id}...")
-                output = manager.install(package_id)
+                logger.info(
+                    f"Install {spec} package with {theme.invoked_command(manager.id)}..."
+                )
+                output = manager.install(spec.package_id, version=spec.version)
                 echo(output)
