@@ -23,7 +23,7 @@ from typing import Iterator
 from click_extra.platform import LINUX
 
 from ..base import Package, PackageManager
-from ..capabilities import search_capabilities
+from ..capabilities import search_capabilities, version_not_implemented
 
 
 class APT(PackageManager):
@@ -201,7 +201,8 @@ class APT(PackageManager):
                 id=package_id, description=description, latest_version=version
             )
 
-    def install(self, package_id: str) -> str:
+    @version_not_implemented
+    def install(self, package_id: str, version: str | None = None) -> str:
         """Install one package.
 
         .. code-block:: shell-session
@@ -220,7 +221,10 @@ class APT(PackageManager):
         """
         return self.build_cli("upgrade", "--yes", sudo=True)
 
-    def upgrade_one_cli(self, package_id: str) -> tuple[str, ...]:
+    @version_not_implemented
+    def upgrade_one_cli(
+        self, package_id: str, version: str | None = None
+    ) -> tuple[str, ...]:
         """Generates the CLI to upgrade all packages (default) or only the one provided
         as parameter.
 
