@@ -21,10 +21,9 @@ from __future__ import annotations
 
 import inspect
 import sys
-from collections.abc import Iterable
 from importlib import import_module
 from pathlib import Path
-from typing import Iterator
+from typing import Iterable, Iterator
 
 if sys.version_info < (3, 8):
     from typing_extensions import Final
@@ -37,6 +36,7 @@ else:
     from boltons.cacheutils import cachedproperty as cached_property
 
 from boltons.iterutils import unique
+from click_extra.colorize import default_theme as theme
 
 from . import logger
 from .base import Operations, PackageManager
@@ -58,7 +58,7 @@ class ManagerPool:
     """Path of files containing package manager definitions."""
 
     @cached_property
-    def register(self):
+    def register(self) -> dict[str, PackageManager]:
         """Search for package manager definitions locally and store them into an
         internal register.
 
@@ -115,7 +115,7 @@ class ManagerPool:
     # Pre-compute all sorts of constants.
 
     @cached_property
-    def all_manager_ids(self):
+    def all_manager_ids(self) -> tuple[str, ...]:
         """All recognized manager IDs.
 
         Returns a list of sorted items to provide consistency across all UI, and
@@ -124,7 +124,7 @@ class ManagerPool:
         return tuple(sorted(self.register))
 
     @cached_property
-    def default_manager_ids(self):
+    def default_manager_ids(self) -> tuple[str, ...]:
         """All manager IDs supported on the current platform.
 
         Must keep the same order defined by :py:prop:`meta_package_manager.pool.ManagerPool.all_manager_ids`.
@@ -134,7 +134,7 @@ class ManagerPool:
         )
 
     @cached_property
-    def unsupported_manager_ids(self):
+    def unsupported_manager_ids(self) -> tuple[str, ...]:
         """All manager IDs unsupported on the current platform.
 
         Order is not important here as this list will be used to discard managers from
@@ -191,13 +191,15 @@ class ManagerPool:
             # call to the package manager CLI.
             if implements_operation and not manager.implements(implements_operation):
                 logger.warning(
-                    f"{manager_id} does not implement {implements_operation}."
+                    f"{theme.invoked_command(manager_id)} does not implement {implements_operation}."
                 )
                 continue
 
             # Filters out inactive managers.
             if drop_inactive and not manager.available:
-                logger.warning(f"Skip unavailable {manager_id} manager.")
+                logger.warning(
+                    f"Skip unavailable {theme.invoked_command(manager_id)} manager."
+                )
                 continue
 
             # Apply manager-level options.
