@@ -48,7 +48,7 @@ class TestInstallRemove(CLISubCommandTests):
         result = invoke(operation)
         assert result.exit_code == 2
         assert not result.stdout
-        assert "Error: Missing argument 'PACKAGE_ID'." in result.stderr
+        assert "Error: Missing argument 'PACKAGES_SPECS...'." in result.stderr
 
     PACKAGE_IDS = {
         "apm": "markdown-pdf",
