@@ -15,6 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import pytest
 from click_extra.tests.conftest import destructive
 
@@ -24,7 +26,7 @@ from .test_cli import CLISubCommandTests
 
 @pytest.fixture
 def subcmd():
-    return "upgrade"
+    return "upgrade", "--all"
 
 
 class TestUpgrade(CLISubCommandTests):
@@ -32,22 +34,40 @@ class TestUpgrade(CLISubCommandTests):
     """All tests here should me marked as destructive unless --dry-run parameter is
     passed."""
 
-    def test_default_all_managers_dry_run(self, invoke, subcmd):
-        result = invoke("--dry-run", subcmd)
+    @pytest.mark.parametrize("all_option", ("--all", None))
+    def test_all_managers_dry_run_upgrade_all(self, invoke, all_option):
+        result = invoke("--dry-run", "upgrade", all_option)
         assert result.exit_code == 0
+        if not all_option:
+            assert "assume -A/--all option" in result.stderr
+        self.check_manager_selection(result)
+
+    @destructive
+    @pytest.mark.parametrize("all_option", ("--all", None))
+    def test_all_managers_upgrade_all(self, invoke, all_option):
+        result = invoke("upgrade", all_option)
+        assert result.exit_code == 0
+        if not all_option:
+            assert "assume -A/--all option" in result.stderr
         self.check_manager_selection(result)
 
     @pytest.mark.parametrize("mid", pool.default_manager_ids)
-    def test_single_manager_dry_run(self, invoke, subcmd, mid):
-        result = invoke(f"--{mid}", "--dry-run", subcmd)
+    @pytest.mark.parametrize("all_option", ("--all", None))
+    def test_single_manager_dry_run_upgrade_all(self, invoke, mid, all_option):
+        result = invoke(f"--{mid}", "--dry-run", "upgrade", all_option)
         assert result.exit_code == 0
+        if not all_option:
+            assert "assume -A/--all option" in result.stderr
         self.check_manager_selection(result, {mid})
 
     @destructive
     @pytest.mark.parametrize("mid", pool.default_manager_ids)
-    def test_single_manager(self, invoke, subcmd, mid):
-        result = invoke(f"--{mid}", subcmd)
+    @pytest.mark.parametrize("all_option", ("--all", None))
+    def test_single_manager_upgrade_all(self, invoke, mid, all_option):
+        result = invoke(f"--{mid}", "upgrade", all_option)
         assert result.exit_code == 0
+        if not all_option:
+            assert "assume -A/--all option" in result.stderr
         self.check_manager_selection(result, {mid})
 
 
