@@ -15,6 +15,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
+from __future__ import annotations
+
 import json
 import re
 import sys
@@ -50,8 +52,8 @@ PROJECT_ROOT = Path(__file__).parent.parent.parent
 
 def test_project_metadata():
     # Fetch general information about the project from pyproject.toml.
-    toml_path = Path(__file__).parent.joinpath("../../pyproject.toml").resolve()
-    toml_config = tomllib.loads(toml_path.read_text())
+    toml_path = PROJECT_ROOT.joinpath("pyproject.toml").resolve()
+    toml_config = tomllib.loads(toml_path.read_text(encoding="utf-8"))
     # Check all managers are referenced in Python package keywords.
     assert set(pool.all_manager_ids).issubset(toml_config["tool"]["poetry"]["keywords"])
 
@@ -171,7 +173,7 @@ def test_labeller_rules():
         # No duplicate labels.
         assert len(set(rule["labels"])) == len(rule["labels"])
 
-        # Special checks for rules targetting manager labels.
+        # Special checks for rules targeting manager labels.
         manager_label = canonical_managers.intersection(rule["labels"])
         if manager_label:
 
